from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsParentStateBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_parent_state"
    section = "draft-parent-record-state"
    template = "drafts-parent-state"

    def finish(self, **extra_kwargs):
        super().finish(parent_record=self.current_model.parent_record, **extra_kwargs)



from oarepo_model_builder.builders.jsonschema import JSONSchemaBuilder
from oarepo_model_builder.utils.dict import dict_get


class JSONSchemaDraftsParentBuilder(JSONSchemaBuilder):
    TYPE = "jsonschema_drafts_parent"
    output_file_type = "jsonschema"
    output_file_name = ["draft-parent-record-schema", "file"]
    parent_module_root_name = "jsonschemas"

    def builtin_json(self):
        return {
            "type": "object",
            "properties": {
                "$schema": {"type": "keyword"},
                "created": {"type": "datetime"},
                "id": {"type": "keyword"},
                "updated": {"type": "datetime"},
            },
        }

    def build_node(self, node):
        from oarepo_model_builder.datatypes import datatypes

        json = self.builtin_json()
        parsed_section = datatypes.get_datatype(
            parent=None,
            data=json,
            key=None,
            model=json,
            schema=json,
        )
        parsed_section.prepare({})
        skip = dict_get(
            self.current_model.definition, ["json-schema-settings", "skip"], False
        )
        if skip:
            return
        generated = self.generate(parsed_section)
        self.output.merge(generated)



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsRecordExtraFieldsBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_record_extra_fields"
    section = "record"
    template = "drafts-record-extra-fields"
    record_section = "section_record"

    def finish(self, **extra_kwargs):
        super().finish(
            record=getattr(self.current_model, self.record_section).config,
            **extra_kwargs,
        )



from oarepo_model_builder.datatypes import DataType, ModelDataType
from oarepo_model_builder.datatypes.components import ServiceModelComponent
from oarepo_model_builder.datatypes.components.model.utils import (
    append_array,
    set_default,
)


class DraftServiceModelComponent(ServiceModelComponent):
    eligible_datatypes = [ModelDataType]
    dependency_remap = ServiceModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if context["profile"] not in {"record", "draft"}:
            return
        record_service = set_default(datatype, "service", {})
        record_service_config = set_default(datatype, "service-config", {})

        if context["profile"] == "draft":
            parent_record_datatype: DataType = context["parent_record"]
            record_service.setdefault(
                "class", parent_record_datatype.definition["service"]["class"]
            )
            record_service_config.setdefault(
                "class", parent_record_datatype.definition["service-config"]["class"]
            )
            record_service_config.setdefault(
                "service-id",
                parent_record_datatype.definition["service-config"]["service-id"],
            )

        if context["profile"] == "record":
            record_service.setdefault(
                "imports",
                [
                    {
                        "import": "invenio_drafts_resources.services.RecordService",
                        "alias": "InvenioRecordService",
                    }
                ],
            )

            record_service_config.setdefault(
                "base-classes",
                ["PermissionsPresetsConfigMixin", "InvenioRecordDraftsServiceConfig"],
            )

            append_array(
                datatype,
                "service-config",
                "imports",
                {
                    "import": "invenio_drafts_resources.services.RecordServiceConfig",
                    "alias": "InvenioRecordDraftsServiceConfig",
                },
            )

        super().before_model_prepare(datatype, context=context, **kwargs)



from oarepo_model_builder.datatypes import ModelDataType
from oarepo_model_builder.datatypes.components.model.record_metadata import (
    RecordMetadataModelComponent,
)
from oarepo_model_builder.datatypes.components.model.utils import (
    append_array,
    place_after,
    set_default,
)


class DraftRecordMetadataModelComponent(RecordMetadataModelComponent):
    eligible_datatypes = [ModelDataType]
    dependency_remap = RecordMetadataModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        draft_metadata = set_default(datatype, "record-metadata", {})
        if context["profile"] in {"record", "draft"}:
            if context["profile"] == "draft":
                draft_metadata.setdefault(
                    "base-classes",
                    ["db.Model", "DraftMetadataBase", "ParentRecordMixin"],
                )
                draft_metadata.setdefault(
                    "imports",
                    [
                        {
                            "import": "invenio_drafts_resources.records.DraftMetadataBase"
                        },
                        {"import": "invenio_db.db"},
                        {
                            "import": "invenio_drafts_resources.records.ParentRecordMixin"
                        },
                    ],
                )
                draft_metadata.setdefault("use-versioning", False)

            super().before_model_prepare(datatype, context=context, **kwargs)

            place_after(
                datatype,
                "record-metadata",
                "base-classes",
                "RecordMetadataBase",
                "ParentRecordMixin",
            )
            append_array(
                datatype,
                "record-metadata",
                "imports",
                {"import": "invenio_drafts_resources.records.ParentRecordMixin"},
            )



from .defaults import DraftDefaultsModelComponent
from .draft_parent import DraftParentComponent
from .jsonschema import DraftJSONSchemaModelComponent
from .mapping import DraftMappingModelComponent
from .pid import DraftPIDModelComponent
from .record import DraftRecordModelComponent
from .record_metadata import DraftRecordMetadataModelComponent
from .resource import DraftResourceModelComponent
from .service import DraftServiceModelComponent

__all__ = [
    "DraftDefaultsModelComponent",
    "DraftPIDModelComponent",
    "DraftRecordModelComponent",
    "DraftRecordMetadataModelComponent",
    "DraftJSONSchemaModelComponent",
    "DraftMappingModelComponent",
    "DraftResourceModelComponent",
    "DraftServiceModelComponent",
    "DraftParentComponent",
]



from .draft import DraftComponent
from .draft_model import (
    DraftDefaultsModelComponent,
    DraftJSONSchemaModelComponent,
    DraftMappingModelComponent,
    DraftParentComponent,
    DraftPIDModelComponent,
    DraftRecordMetadataModelComponent,
    DraftRecordModelComponent,
    DraftResourceModelComponent,
    DraftServiceModelComponent,
)
from .draft_tests import DraftModelTestComponent

DRAFT_COMPONENTS = [
    DraftParentComponent,
    DraftComponent,
    DraftRecordModelComponent,
    DraftRecordMetadataModelComponent,
    DraftPIDModelComponent,
    DraftDefaultsModelComponent,
    DraftJSONSchemaModelComponent,
    DraftModelTestComponent,
    DraftMappingModelComponent,
    DraftResourceModelComponent,
    DraftServiceModelComponent,
]



from oarepo_model_builder.datatypes import ModelDataType
from oarepo_model_builder.datatypes.components import PIDModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default


class DraftPIDModelComponent(PIDModelComponent):
    eligible_datatypes = [ModelDataType]
    dependency_remap = PIDModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if context["profile"] not in {"record", "draft"}:
            return

        pid = set_default(datatype, "pid", {})
        pid.setdefault("provider-base-classes", ["DraftRecordIdProviderV2"])
        pid.setdefault(
            "imports",
            [
                {
                    "import": "invenio_records_resources.records.systemfields.pid.PIDField"
                },
                {
                    "import": "invenio_records_resources.records.systemfields.pid.PIDFieldContext"
                },
                {
                    "import": "invenio_drafts_resources.records.api.DraftRecordIdProviderV2"
                },
            ],
        )
        if context["profile"] == "draft":
            parent = context["parent_record"]
            pid = set_default(datatype, "pid", {})
            pid.setdefault(
                "provider-class",
                parent.definition["pid"]["provider-class"],
            )
            pid.setdefault("field-args", ["create=True", "delete=False"])

        super().before_model_prepare(datatype, context=context, **kwargs)



from oarepo_model_builder.datatypes import DataTypeComponent, ModelDataType
from oarepo_model_builder_tests.datatypes.components import ModelTestComponent


class DraftModelTestComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    depends_on = [ModelTestComponent]

    def process_tests(self, datatype, section, **extra_kwargs):
        section.fixtures = {
            "record_service": "record_service",
            "sample_record": "sample_draft",
        }

        section.constants = {
            "read_url": "/draft",
            "update_url": "/draft",
            "delete_url": "/draft",
            "deleted_http_code": 404,
            "skip_search_test": True,
            "service_read_method": "read_draft",
            "service_create_method": "create",
            "service_delete_method": "delete_draft",
            "service_update_method": "update_draft",
            "deleted_record_pid_error": "PIDDoesNotExistError",
            "revision_id1": 2,
            "revision_id2": 5,
            "revision_id3": 8,
        }



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsHasDraftCheckfieldBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_has_draft_checkfield"
    section = "record"
    template = "drafts-has-draft-checkfield"

    def finish(self, **extra_kwargs):
        super().finish(parent_record=self.current_model.parent_record, **extra_kwargs)



TEMPLATES = {
    "drafts-record-extra-fields": "templates/invenio_drafts_record_extra_fields.py.jinja2",  # adds drafts extension fields to record
    "drafts-record-metadata-extra-fields": "templates/invenio_drafts_record_metadata_extra_fields.py.jinja2",  # adds parent specific fields to model metadata classes
    "drafts-parent-record": "templates/invenio_drafts_parent_record.py.jinja2",  # adds draft parent record class
    "drafts-parent-metadata": "templates/invenio_drafts_parent_metadata.py.jinja2",  # adds draft parent metadata classes
    "drafts-record-service-config": "templates/invenio_drafts_record_service_config.py.jinja2",
    "drafts-parent-state": "templates/invenio_drafts_parent_state.py.jinja2",
    "drafts-has-draft-checkfield": "templates/invenio_drafts_has_draft_checkfield.py.jinja2",
}



from oarepo_model_builder.datatypes import DataTypeComponent, datatypes

from oarepo_model_builder_drafts.datatypes import DraftDataType


class DraftMappingModelComponent(DataTypeComponent):
    eligible_datatypes = [DraftDataType]

    def process_mapping(self, datatype, section, **kwargs):
        if self.is_draft_profile:  # this will work but idk if it's correct approach
            section.children["expires_at"] = datatypes.get_datatype(
                datatype,
                {
                    "type": "datetime",
                    "sample": {"skip": True},
                    "marshmallow": {"read": False, "write": False},
                    "ui": {"marshmallow": {"read": False, "write": False}},
                },
                "expires_at",
                datatype.model,
                datatype.schema,
            )
            section.children["expires_at"].prepare(context={})
            section.children["fork_version_id"] = datatypes.get_datatype(
                datatype,
                {
                    "type": "integer",
                    "sample": {"skip": True},
                    "marshmallow": {"read": False, "write": False},
                    "ui": {"marshmallow": {"read": False, "write": False}},
                },
                "fork_version_id",
                datatype.model,
                datatype.schema,
            )
            section.children["fork_version_id"].prepare(context={})

    def before_model_prepare(self, datatype, *, context, **kwargs):
        self.is_draft_profile = context["profile"] == "draft"



from oarepo_model_builder.datatypes.components import DefaultsModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default

from oarepo_model_builder_drafts.datatypes import DraftDataType


class DraftDefaultsModelComponent(DefaultsModelComponent):
    eligible_datatypes = [DraftDataType]
    dependency_remap = DefaultsModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        parent = context["parent_record"]
        set_default(datatype, "model-name", parent.definition["model-name"])
        module_container = set_default(datatype, "module", {})
        module_container.setdefault(
            "prefix", f"{parent.definition['module']['prefix']}Draft"
        )

        super().before_model_prepare(datatype, context=context, **kwargs)



from pathlib import Path
from typing import List, Union

from oarepo_model_builder.builder import ModelBuilder
from oarepo_model_builder.profiles.record import RecordProfile
from oarepo_model_builder.schema import ModelSchema
from oarepo_model_builder.utils.dict import dict_get


class DraftProfile(RecordProfile):
    default_model_path = ["record", "draft"]

    def build(
        self,
        model: ModelSchema,
        profile: str,
        model_path: List[str],
        output_directory: Union[str, Path],
        builder: ModelBuilder,
        **kwargs,
    ):
        parent_record = model.get_schema_section("record", model_path[:-1])
        if not dict_get(model.schema, model_path):
            return
        draft_profile = dict_get(model.schema, model_path)
        draft_profile["type"] = "draft_record"

        # pass the parent record as an extra context item. This will be handled by file-aware
        # components in their "prepare" method
        super().build(
            model=model,
            profile=profile,
            model_path=model_path,
            output_directory=output_directory,
            builder=builder,
            context={
                "profile": "draft",
                "profile_module": "records",
                "parent_record": parent_record,
            },
        )



from oarepo_model_builder.datatypes.components import JSONSchemaModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default

from oarepo_model_builder_drafts.datatypes import DraftDataType


class DraftJSONSchemaModelComponent(JSONSchemaModelComponent):
    eligible_datatypes = [DraftDataType]
    dependency_remap = JSONSchemaModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        parent_datatype = context["parent_record"].definition

        json_schema = set_default(datatype, "json-schema-settings", {})
        json_schema.setdefault(
            "alias", parent_datatype["json-schema-settings"]["alias"]
        )

        json_schema.setdefault(
            "module",
            parent_datatype["json-schema-settings"]["module"],
        )
        json_schema.setdefault(
            "name",
            parent_datatype["json-schema-settings"]["name"],
        )

        json_schema.setdefault(
            "file",
            parent_datatype["json-schema-settings"]["file"],
        )

        super().before_model_prepare(datatype, context=context, **kwargs)



import os

import marshmallow as ma
from oarepo_model_builder.datatypes import DataTypeComponent, ModelDataType
from oarepo_model_builder.datatypes.components import (
    RecordMetadataModelComponent,
    RecordModelComponent,
)
from oarepo_model_builder.datatypes.components.model.utils import set_default
from oarepo_model_builder.utils.python_name import module_to_path, parent_module
from oarepo_model_builder.validation.utils import ImportSchema


class DraftParentRecordSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    class_ = ma.fields.Str(
        attribute="class",
        data_key="class",
        metadata={"doc": "Qualified name of the class"},
    )
    base_classes = ma.fields.List(
        ma.fields.Str(),
        attribute="base-classes",
        data_key="base-classes",
        metadata={"doc": "Model base classes"},
    )
    extra_code = ma.fields.Str(
        attribute="extra-code",
        data_key="extra-code",
        metadata={"doc": "Extra code to copy to record file"},
    )
    module = ma.fields.String(metadata={"doc": "Class module"})
    imports = ma.fields.List(
        ma.fields.Nested(ImportSchema), metadata={"doc": "List of python imports"}
    )
    skip = ma.fields.Boolean()
    generate = ma.fields.Boolean()


class DraftParentRecordStateSchema(ma.Schema):
    class Meta:
        unknown = ma.RAISE

    class_ = ma.fields.Str(
        attribute="class",
        data_key="class",
        metadata={"doc": "Qualified name of the class"},
    )
    base_classes = ma.fields.List(
        ma.fields.Str(),
        attribute="base-classes",
        data_key="base-classes",
        metadata={"doc": "Model base classes"},
    )

    imports = ma.fields.List(
        ma.fields.Nested(ImportSchema), metadata={"doc": "List of python imports"}
    )
    module = ma.fields.String(
        metadata={"doc": "Module where the facets will be placed"}
    )
    table = ma.fields.String()
    generate = ma.fields.Boolean()
    skip = ma.fields.Boolean()


class DraftParentRecordMetadataSchema:
    class Meta:
        unknown = ma.RAISE

    class_ = ma.fields.Str(
        attribute="class",
        data_key="class",
        metadata={"doc": "Qualified name of the class"},
    )
    base_classes = ma.fields.List(
        ma.fields.Str(),
        attribute="base-classes",
        data_key="base-classes",
        metadata={"doc": "Model base classes"},
    )

    imports = ma.fields.List(
        ma.fields.Nested(ImportSchema), metadata={"doc": "List of python imports"}
    )
    module = ma.fields.String(
        metadata={"doc": "Module where the facets will be placed"}
    )
    table = ma.fields.String()
    generate = ma.fields.Boolean()
    skip = ma.fields.Boolean()


class DraftParentRecordJsonSchema:
    name = ma.fields.Str(metadata={"doc": "Schema name"})
    module = ma.fields.Str(metadata={"doc": "Schema module"})
    file_ = ma.fields.Str(
        data_key="file", attribute="file", metadata={"doc": "Path to schema file"}
    )
    generate = ma.fields.Boolean()
    skip = ma.fields.Boolean()


class DraftParentComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    depends_on = [RecordModelComponent, RecordMetadataModelComponent]

    class ModelSchema(ma.Schema):
        draft_parent_record = ma.fields.Nested(
            DraftParentRecordSchema,
            attribute="draft-parent-record",
            data_key="draft-parent-record",
            # metadata={"doc": "Service config settings"},
        )
        draft_parent_record_state = ma.fields.Nested(
            DraftParentRecordStateSchema,
            attribute="draft-parent-record-state",
            data_key="draft-parent-record-state",
            # metadata={"doc": "Service config settings"},
        )
        draft_parent_record_metadata = ma.fields.Nested(
            DraftParentRecordMetadataSchema,
            attribute="draft-parent-record-metadata",
            data_key="draft-parent-record-metadata",
            # metadata={"doc": "Service config settings"},
        )
        draft_parent_record_json_schema = ma.fields.Nested(
            DraftParentRecordJsonSchema,
            attribute="draft-parent-record-schema",
            data_key="draft-parent-record-schema",
            # metadata={"doc": "Service config settings"},
        )

    def before_model_prepare(self, datatype, *, context, **kwargs):
        record_module = datatype.definition["record"]["module"]
        metadata_module = datatype.definition["record-metadata"]["module"]
        record_prefix = datatype.definition["module"]["prefix"]
        records_path = module_to_path(
            parent_module(datatype.definition["record"]["module"])
        )
        is_record = context["profile"] == "record"
        if not is_record:
            parent_record_datatype = context["parent_record"]

        draft_parent_record = set_default(datatype, "draft-parent-record", {})
        if is_record:
            draft_parent_record.setdefault(
                "class", f"{record_module}.{record_prefix}ParentRecord"
            )
        else:
            draft_parent_record.setdefault(
                "class",
                parent_record_datatype.definition["draft-parent-record"]["class"],
            )
        draft_parent_record.setdefault("base-classes", ["ParentRecord"])
        draft_parent_record.setdefault(
            "imports", [{"import": "invenio_drafts_resources.records.api.ParentRecord"}]
        )
        draft_parent_record.setdefault("module", record_module)
        draft_parent_record.setdefault("generate", True)

        draft_parent_record_state = set_default(
            datatype, "draft-parent-record-state", {}
        )
        if is_record:
            draft_parent_record_state.setdefault(
                "class", f"{metadata_module}.{record_prefix}ParentState"
            )
        else:
            draft_parent_record_state.setdefault(
                "class",
                parent_record_datatype.definition["draft-parent-record-state"]["class"],
            )
        draft_parent_record_state.setdefault(
            "base-classes", ["db.Model", "ParentRecordStateMixin"]
        )
        draft_parent_record_state.setdefault(
            "imports",
            [
                {"import": "invenio_drafts_resources.records.ParentRecordStateMixin"},
                {"import": "invenio_db.db"},
            ],
        )
        draft_parent_record_state.setdefault("module", metadata_module)
        draft_parent_record_state.setdefault("generate", True)
        draft_parent_record_state.setdefault(
            "table",
            f"{datatype.definition['module']['qualified']}_parent_state_metadata",
        )

        draft_parent_record_metadata = set_default(
            datatype, "draft-parent-record-metadata", {}
        )
        if is_record:
            draft_parent_record_metadata.setdefault(
                "class", f"{metadata_module}.{record_prefix}ParentMetadata"
            )
        else:
            draft_parent_record_metadata.setdefault(
                "class",
                parent_record_datatype.definition["draft-parent-record-metadata"][
                    "class"
                ],
            )
        draft_parent_record_metadata.setdefault(
            "base-classes", ["db.Model", "RecordMetadataBase"]
        )
        draft_parent_record_metadata.setdefault(
            "imports",
            [
                {"import": "invenio_records.models.RecordMetadataBase"},
                {"import": "invenio_db.db"},
            ],
        )
        draft_parent_record_metadata.setdefault(
            "table",
            f"{datatype.definition['module']['qualified']}_parent_record_metadata",
        )
        draft_parent_record_metadata.setdefault("module", metadata_module)
        draft_parent_record_metadata.setdefault("generate", True)

        json_schema = set_default(datatype, "draft-parent-record-schema", {})
        schema_name = json_schema.setdefault(
            "name",
            f"parent-v1.0.0.json",
        )

        json_schema.setdefault(
            "file", os.path.join(records_path, "jsonschemas", schema_name)
        )
        json_schema.setdefault("generate", True)



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsConftestBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_conftest"
    template = "drafts-conftest"
    MODULE = "tests.conftest"

    def _get_output_module(self):
        return f'{self.current_model.definition["tests"]["module"]}.conftest'

    def finish(self, **extra_kwargs):
        tests = getattr(self.current_model, "section_tests")
        super().finish(
            fixtures=tests.fixtures, test_constants=tests.constants, **extra_kwargs
        )



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsRecordMetadataExtraFieldsBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_record_metadata_extra_fields"
    section = "record-metadata"
    template = "drafts-record-metadata-extra-fields"
    record_metadata_section = "section_record_metadata"

    def finish(self, **extra_kwargs):
        super().finish(
            record_metadata=getattr(
                self.current_model, self.record_metadata_section
            ).config,
            **extra_kwargs,
        )



from pathlib import Path

from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder
from oarepo_model_builder.outputs.python import PythonOutput


class InvenioDraftsParentRecordBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_parent_record"
    section = "draft-parent-record"
    template = "drafts-parent-record"

    def process_template(self, python_path: Path, template, **extra_kwargs):
        if self.parent_modules:
            self.create_parent_modules(python_path)
        output: PythonOutput = self.builder.get_output("python", python_path)
        context = dict(
            settings=self.settings,
            current_model=self.current_model,
            schema=self.current_model.schema.schema,
            **extra_kwargs,
        )
        output.merge(template, context)



from oarepo_model_builder.datatypes import DataType, ModelDataType
from oarepo_model_builder.datatypes.components import ResourceModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default


class DraftResourceModelComponent(ResourceModelComponent):
    eligible_datatypes = [ModelDataType]
    dependency_remap = ResourceModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if context["profile"] not in {"record", "draft"}:
            return
        record_resource = set_default(datatype, "resource", {})
        record_resource_config = set_default(datatype, "resource-config", {})

        if context["profile"] == "draft":
            parent_record_datatype: DataType = context["parent_record"]
            record_resource.setdefault(
                "class", parent_record_datatype.definition["resource"]["class"]
            )
            record_resource_config.setdefault(
                "class", parent_record_datatype.definition["resource-config"]["class"]
            )

        if context["profile"] == "record":
            record_resource.setdefault(
                "imports",
                [{"import": "invenio_drafts_resources.resources.RecordResource"}],
            )
            record_resource_config.setdefault(
                "imports",
                [{"import": "invenio_drafts_resources.resources.RecordResourceConfig"}],
            )

        super().before_model_prepare(datatype, context=context, **kwargs)



import marshmallow as ma
from oarepo_model_builder.datatypes import ModelDataType


class DraftDataType(ModelDataType):
    model_type = "draft_record"

    class ModelSchema(ModelDataType.ModelSchema):
        type = ma.fields.Str(
            load_default="draft_record",
            required=False,
            validate=ma.validate.Equal("draft_record"),
        )

    def prepare(self, context):
        self.parent_record = context["parent_record"]
        super().prepare(context)



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsParentMetadataBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_parent_metadata"
    section = "draft-parent-record-metadata"
    template = "drafts-parent-metadata"



from oarepo_model_builder.datatypes import ModelDataType
from oarepo_model_builder.datatypes.components import RecordModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default


class DraftRecordModelComponent(RecordModelComponent):
    eligible_datatypes = [ModelDataType]
    dependency_remap = RecordModelComponent

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if context["profile"] not in {"record", "draft"}:
            return
        record = set_default(datatype, "record", {})

        if context["profile"] == "draft":
            record.setdefault("base-classes", ["InvenioDraft"])
            record.setdefault(
                "imports",
                [
                    {
                        "import": "invenio_drafts_resources.records.api.Draft",
                        "alias": "InvenioDraft",
                    }
                ],
            )
            is_record_preset = record.get("class", None)
            super().before_model_prepare(datatype, context=context, **kwargs)
            if not is_record_preset and record["class"][-6:] == "Record":
                record["class"] = record["class"][:-6]

        if context["profile"] == "record":
            record.setdefault(
                "imports",
                [
                    {
                        "import": "invenio_drafts_resources.records.api.Record",
                        "alias": "InvenioRecord",
                    }
                ],
            )
            super().before_model_prepare(datatype, context=context, **kwargs)



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsTestResourcesBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_test_resources"
    template = "drafts-test-resources"
    MODULE = "tests.test_resources"

    def _get_output_module(self):
        return f'{self.current_model.definition["tests"]["module"]}.test_resource'

    def finish(self, **extra_kwargs):
        tests = getattr(self.current_model, "section_tests")
        super().finish(
            fixtures=tests.fixtures,
            test_constants=tests.constants,
            parent_record=self.current_model.parent_record,
            **extra_kwargs,
        )



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsRecordServiceConfigBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_record_service_config"
    section = "service-config"
    template = "drafts-record-service-config"

    def finish(self, **extra_kwargs):
        super().finish(parent_record=self.current_model.parent_record, **extra_kwargs)



import marshmallow as ma
from oarepo_model_builder.datatypes import (
    DataType,
    DataTypeComponent,
    Import,
    ModelDataType,
    Section,
)
from oarepo_model_builder.datatypes.components import DefaultsModelComponent
from oarepo_model_builder.datatypes.components.model.utils import set_default
from oarepo_model_builder.datatypes.model import Link


def get_draft_schema():
    from ..draft import DraftDataType

    return DraftDataType.validator()


class DraftComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    affects = [DefaultsModelComponent]

    class ModelSchema(ma.Schema):
        draft = ma.fields.Nested(get_draft_schema)

    def process_links(self, datatype, section: Section, **kwargs):
        # add files link item
        if self.is_record_profile:
            if "links_search" in section.config:
                section.config.pop("links_search")
            for link in section.config["links_item"]:
                if link.name == "self":
                    section.config["links_item"].remove(link)
                    break
            section.config["links_item"] += [
                Link(
                    name="self",
                    link_class="ConditionalLink",
                    link_args=[
                        "cond=is_record",
                        'if_=RecordLink("{+api}{self.url_prefix}{id}")',
                        'else_=RecordLink("{+api}{self.url_prefix}{id}/draft")',
                    ],
                    imports=[
                        Import("invenio_records_resources.services.ConditionalLink"),
                        Import("invenio_records_resources.services.RecordLink"),
                        Import(
                            "invenio_drafts_resources.services.records.config.is_record"
                        ),
                    ],
                ),
                Link(
                    name="self_html",
                    link_class="ConditionalLink",
                    link_args=[
                        "cond=is_record",
                        'if_=RecordLink("{+ui}{self.url_prefix}{id}")',
                        'else_=RecordLink("{+ui}/uploads/{id}")',
                    ],
                    imports=[
                        Import("invenio_records_resources.services.ConditionalLink"),
                        Import("invenio_records_resources.services.RecordLink"),
                        Import(
                            "invenio_drafts_resources.services.records.config.is_record"
                        ),
                    ],
                ),
                Link(
                    name="latest",
                    link_class="RecordLink",
                    link_args=['"{+api}/{self.url_prefix}{id}/versions/latest"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
                Link(
                    name="latest_html",
                    link_class="RecordLink",
                    link_args=['"{+ui}/{self.url_prefix}{id}/latest"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
                Link(
                    name="draft",
                    link_class="RecordLink",
                    link_args=['"{+api}/{self.url_prefix}{id}/draft"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
                Link(
                    name="record",
                    link_class="RecordLink",
                    link_args=['"{+api}/{self.url_prefix}{id}"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
                Link(
                    name="publish",
                    link_class="RecordLink",
                    link_args=['"{+api}/{self.url_prefix}{id}/draft/actions/publish"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
                Link(
                    name="versions",
                    link_class="RecordLink",
                    link_args=['"{+api}/{self.url_prefix}{id}/versions"'],
                    imports=[Import("invenio_records_resources.services.RecordLink")],
                ),
            ]

    def before_model_prepare(self, datatype, *, context, **kwargs):
        self.is_draft_profile = context["profile"] == "draft"
        self.is_record_profile = context["profile"] == "record"
        if self.is_draft_profile:
            parent_record_datatype: DataType = context["parent_record"]
            datatype.parent_record = parent_record_datatype

            properties = set_default(datatype, "properties", {})
            for property_key, property_value in parent_record_datatype.definition[
                "properties"
            ].items():  # this should
                properties.setdefault(property_key, property_value)
